library RAM;

uses
  SysUtils,
  Windows,
  Classes,
  Controls;

{$R *.RES}

Const  Inputs   = 17;        // number of inputs
       Outputs  = 8;         // number of outputs made adjustable
       MAXCOUNT = 96;        //anstelle von 100;

       {INPUTS}
       A0    = 0;            // index
       A1    = 1;
       A2    = 2;
       A3    = 3;
       A4    = 4;
       A5    = 5;
       A6    = 6;
       D0    = 7;
       D1    = 8;
       D2    = 9;
       D3    = 10;
       D4    = 11;
       D5    = 12;
       D6    = 13;
       D7    = 14;
       CLK   = 15;            // index of input variable CLK
       RST   = 16;            // index of input variable RST

       {OUTPUTS}
       Q0    = 0;
       Q1    = 1;
       Q2    = 2;
       Q3    = 3;
       Q4    = 4;
       Q5    = 5;
       Q6    = 6;
       Q7    = 7;

       {USER}
       CLK_old = 0;          // index of user variable CLK_old
       RST_old = 1;          // index of user variable RST_old
       Count   = 2;          // index of user variable COUNT
       DB0     = 3;


       DLLIndex = 100;       // fixed index of DLL No. (set by ProfiLab)

Type TDLLParams = array[0..100] of extended;   //Type of ProfiLab DLL parameters
     PDLLParams = ^TDLLParams;                 // Pointer to ProfiLab DLL parameters

function NumInputs: Byte;
begin
  result:=Inputs; //Define FIXED number of component input pins
end;

function NumOutputs: Byte; // Made adjustable with 'NumOutputsEx'
begin
  result:=Outputs; //Define FIXED number of component output pins
end;

{function NumInputsEx(PUser: PDLLParams): Byte; //Just in case number of inputs need to be variable
begin
  result:=...; //Define VARIABLE number of component input pins
end;}
{
function NumOutputsEx(PUser: PDLLParams): Byte;
begin
  result:= 8; //8 bit
end;
}
Function InputName(Channel: Byte): ShortString; // Return name for each component input pin
begin
  case Channel of
  CLK: result:='/CLK';   //  "CLK" (Clock)
  RST: result:='/RST';  //  "/RST" (NOT RESET)
  D0 : result:='D0';
  D1 : result:='D1';
  D2 : result:='D2';
  D3 : result:='D3';
  D4 : result:='D4';
  D5 : result:='D5';
  D6 : result:='D6';
  D7 : result:='D7';
  A0 : result:='A0';
  A1 : result:='A1';
  A2 : result:='A2';
  A3 : result:='A3';
  A4 : result:='A4';
  A5 : result:='A5';
  A6 : result:='A6';
  end;
end;

Function OutputName(Channel: Byte): ShortString; // Return name for each component output pin
begin
  case Channel of
  Q0 : result:='Q0';
  Q1 : result:='Q1';
  Q2 : result:='Q2';
  Q3 : result:='Q3';
  Q4 : result:='Q4';
  Q5 : result:='Q5';
  Q6 : result:='Q6';
  Q7 : result:='Q7';
  end;
end;

Procedure SimStart(PInput,POutput,PUser: PDLLParams); //called when ProfiLab enters RUN mode
var i: Integer;
begin
  PUser^[Count]:= 0; //RESET COUNTER
  PUser^[RST_old]:= 5;
  PUser^[CLK_old]:= 5;

  For i:= 0 to 7 do
  begin
    POutput^[i]:= 0; //Set binary outputs
  end;

  //Speicher loeschen
  For i:= 0 to MAXCOUNT-1 do
  begin
    PUser^[DB0 + i]:= 0;
  end;
end;

Procedure SimStop(PInput,POutput,PUser: PDLLParams); //called when ProfiLab RUN mode is terminated
begin
  // nothing to be done
end;

Procedure Calculate(PInput,POutput,PUser: PDLLParams); //called regularly from ProfiLab
var i : Integer;
    Adresse,Din,Dout : word;
    OVR : boolean;
begin
  //Adresse
  Adresse:= 0;
  if (PInput^[A0] > 2.5) then Adresse:= Adresse or $01;
  if (PInput^[A1] > 2.5) then Adresse:= Adresse or $02;
  if (PInput^[A2] > 2.5) then Adresse:= Adresse or $04;
  if (PInput^[A3] > 2.5) then Adresse:= Adresse or $08;
  if (PInput^[A4] > 2.5) then Adresse:= Adresse or $10;
  if (PInput^[A5] > 2.5) then Adresse:= Adresse or $20;
  if (PInput^[A6] > 2.5) then Adresse:= Adresse or $40;
  if Adresse >= MaxCount then
  begin
    Adresse:= MaxCount-1;
    OVR:= true;
  end
  else OVR:= false;

  //Din
  Din:= 0;
  if (PInput^[D0] > 2.5) then Din:= Din or $01;
  if (PInput^[D1] > 2.5) then Din:= Din or $02;
  if (PInput^[D2] > 2.5) then Din:= Din or $04;
  if (PInput^[D3] > 2.5) then Din:= Din or $08;
  if (PInput^[D4] > 2.5) then Din:= Din or $10;
  if (PInput^[D5] > 2.5) then Din:= Din or $20;
  if (PInput^[D6] > 2.5) then Din:= Din or $40;
  if (PInput^[D7] > 2.5) then Din:= Din or $80;

  //RESET
  if PInput^[RST] < 2.5 then //check RST input HIGH or LOW
  begin
    if (not (PInput^[RST] >= 2.5)) and (PUser^[RST_old] > 2.5) then //check out falling edge at RST input
    begin
      PUser^[Count]:= 0; //RESET COUNT
      //Speicher loeschen
      For i:= 0 to MAXCOUNT-1 do
      begin
        PUser^[DB0 + i]:= 0;
      end;
    end;
    exit;
  end;
  PUser^[RST_old]:= PInput^[RST]; //Remember RST status for next call

  //Setze Wert
  if PInput^[CLK] < 2.5 then //check CLK input HIGH or LOW
  begin
    if (not(PInput^[CLK] >= 2.5)) and (PUser^[CLK_old] > 2.5) then //check out rising edge at CLK input
    begin
      if PUser^[Count] > MaxCount-1 then
      begin
        //Speicher schieben
        for i:= 0 to MaxCount-2 do
        begin
          PUser^[DB0 + i]:= PUser^[DB0 + i+1];
        end;
        PUser^[DB0 + MaxCount-1]:= Din; //hinten anfuegen
      end
      else begin
             i:= round(PUser^[count]);
             PUser^[DB0 +i]:= Din;
             PUser^[Count]:= PUser^[Count] +1; // increase COUNT
           end;
    end;
  end;
  PUser^[CLK_old]:=PInput^[CLK]; //Remember CLK status for next call

  //Dout
  Dout:= round(PUser^[DB0 + Adresse]);
  if OVR then Dout:= 0; //PUser^[DB0 + MaxCount-1]

  For i:= 0 to 7 do
  begin
    if (Dout and (1 shl i)) > 0 then POutput^[i]:= 5
                                else POutput^[i]:= 0; //Set binary outputs
  end;
end;




Procedure Configure(PUser: PDLLParams);
begin
//
end;


//export methods for ProfiLab
exports SimStart,
        SimStop,
        NumInputs,
        NumOutputs,
        Calculate,
        InputName,
        OutputName,
        Configure;
begin
end.


